@@ -1,6 +1,6 @@
 #!/usr/bin/env python2
 
-import sys, os, subprocess, string, re, json, threading, multiprocessing
+import sys, os, subprocess, string, re, json, threading, multiprocessing, argparse
 from threading import Thread
 from sys import platform as _platform
 
@@ -33,6 +33,8 @@ class Test:
         self.must_fail = False
         self.blacklist_platforms = []
         self.qt4compat = False
+        self.only_qt = False
+        self.qt_developer = False
 
     def isScript(self):
         return self.filename.endswith(".sh")
@@ -145,6 +147,10 @@ def load_json(check_name):
                 test.expects_failure = t['expects_failure']
             if 'qt4compat' in t:
                 test.qt4compat = t['qt4compat']
+            if 'only_qt' in t:
+                test.only_qt = t['only_qt']
+            if 'qt_developer' in t:
+                test.qt_developer = t['qt_developer']
 
             if not test.checks:
                 test.checks.append(test.check.name)
@@ -205,6 +211,12 @@ def clazy_standalone_command(test, qt):
     if test.qt4compat:
         result = " -qt4-compat " + result
 
+    if test.only_qt:
+        result = " -only-qt " + result
+
+    if test.qt_developer:
+        result = " -qt-developer " + result
+
     return result
 
 def clazy_command(qt, test, filename):
@@ -219,6 +231,12 @@ def clazy_command(qt, test, filename):
     if test.qt4compat:
         result = result + " -Xclang -plugin-arg-clang-lazy -Xclang qt4-compat "
 
+    if test.only_qt:
+        result = result + " -Xclang -plugin-arg-clang-lazy -Xclang only-qt "
+
+    if test.qt_developer:
+        result = result + " -Xclang -plugin-arg-clang-lazy -Xclang qt-developer "
+
     if test.link and _platform.startswith('linux'): # Linking on one platform is enough. Won't waste time on macOS and Windows.
         result = result + " " + link_flags()
     else:
@@ -252,16 +270,29 @@ except:
 
 CLANG_VERSION = int(version.replace('.', ''))
 
+#-------------------------------------------------------------------------------
+# Setup argparse
+
+parser = argparse.ArgumentParser()
+parser.add_argument("-v", "--verbose", action='store_true')
+parser.add_argument("--no-standalone", action='store_true', help="Don\'t run clazy-standalone")
+parser.add_argument("--only-standalone", action='store_true', help='Only run clazy-standalone')
+parser.add_argument("--dump-ast", action='store_true', help='Dump a unit-test AST to file')
+parser.add_argument("check_names", nargs='*', help="The name of the check who's unit-tests will be run. Defaults to running all checks.")
+args = parser.parse_args()
+
+if args.only_standalone and args.no_standalone:
+    print "Error: --only-standalone is incompatible with --no-standalone"
+    sys.exit(1)
+
 #-------------------------------------------------------------------------------
 # Global variables
 
 _enable_fixits_argument = "-Xclang -plugin-arg-clang-lazy -Xclang enable-all-fixits"
-_help_command = "echo | clang -Xclang -load -Xclang " + libraryName() + " -Xclang -add-plugin -Xclang clang-lazy -Xclang -plugin-arg-clang-lazy -Xclang help -c -xc -"
-_dump_ast = "--dump-ast" in sys.argv
-_verbose = "--verbose" in sys.argv
-_help = "--help" in sys.argv
-_no_standalone = "--no-standalone" in sys.argv
-_only_standalone = "--only-standalone" in sys.argv
+_dump_ast = args.dump_ast
+_verbose = args.verbose
+_no_standalone = args.no_standalone
+_only_standalone = args.only_standalone
 _num_threads = multiprocessing.cpu_count()
 _lock = threading.Lock()
 _was_successful = True
@@ -301,17 +332,6 @@ def run_command(cmd, output_file = "", test_env = os.environ):
 
     return success
 
-def print_usage():
-    print "Usage for " + sys.argv[0].strip("./") + ":\n"
-    print "    " + sys.argv[0] + " [--help] [--dump-ast] [check1,check2,check3]"
-    print
-    print "    Without any check supplied, all checks will be run."
-    print "    --dump-ast is provided for debugging purposes.\n"
-    print "Help for clang plugin:"
-    print
-
-    run_command(_help_command)
-
 def files_are_equal(file1, file2):
     try:
         f = open(file1, 'r')
@@ -334,6 +354,7 @@ def get_fixed_files():
     return filter(lambda entry: entry.endswith('.cpp_fixed.cpp'), os.listdir("."))
 
 def print_differences(file1, file2):
+    # Returns true if the the files are equal
     return run_command("diff -Naur {} {}".format(file1, file2))
 
 def normalizedCwd():
@@ -410,17 +431,17 @@ def run_unit_test(test, is_standalone):
     if test.expects_failure:
         if success:
             print "[XOK]   " + printableName
+            return False
         else:
             print "[XFAIL] " + printableName
-            if not print_differences(expected_file, result_file):
-                return False
+            print_differences(expected_file, result_file)
     else:
         if success:
             print "[OK]   " + printableName
         else:
             print "[FAIL] " + printableName
-            if not print_differences(expected_file, result_file):
-                return False
+            print_differences(expected_file, result_file)
+            return False
 
     return True
 
@@ -458,22 +479,14 @@ def load_checks(all_check_names):
 #-------------------------------------------------------------------------------
 # main
 
-if _help:
-    print_usage()
-    sys.exit(0)
-
-args = sys.argv[1:]
-
-switches = ["--verbose", "--dump-ast", "--help", "--no-standalone", "--only-standalone"]
-
-if _dump_ast:
-    del(args[args.index("--dump-ast")])
+if 'CLAZY_NO_WERROR' in os.environ:
+    del os.environ['CLAZY_NO_WERROR']
 
 os.environ['CLAZY_CHECKS'] = ''
 
 all_check_names = get_check_names()
 all_checks = load_checks(all_check_names)
-requested_check_names = filter(lambda x: x not in switches, args)
+requested_check_names = args.check_names
 requested_check_names = map(lambda x: x.strip("/\\"), requested_check_names)
 
 for check_name in requested_check_names:
